import numpy as np

REFEREE_GOAL_COLORS = [
    (237, 237, 237),
    (20, 20, 20),
    (16, 16, 188),
    (211, 211, 0),
    (9, 239, 239),
    (99, 205, 255),
    (22, 201, 16),
]


def dist_between_colors(c1, c2):
    return np.linalg.norm(np.array(c1) - np.array(c2))


def choose_compatible_colors(color_options, home_team_color, away_team_color):
    """
    Take the three colors from color_options that are farest from teams colors
    """

    def key(color):
        dist_home = dist_between_colors(color, home_team_color)
        dist_away = dist_between_colors(color, away_team_color)
        return dist_home + dist_away

    return sorted(color_options, key=key)[-3:]


def hex_to_bgr(hex_color):
    return tuple(int(hex_color.lstrip("#")[i : i + 2], 16) for i in (4, 2, 0))


def get_team_colors(match, order="bgr"):
    """
    Args:
        * match: match data (usually from the api/database)
        * order: order of the colors, should be either 'bgr' or 'rgb'
    Returns:
        * team_colors: dict of colors (in BGR) of teams jersey and short
    """
    if order not in {"bgr", "rgb"}:
        raise ValueError('order should be either "bgr" or "rgb"')
    team_colors = {}
    for side in ["home", "away"]:
        kit = match[f"{side}_team_kit"]
        for kind in ["jersey", "number"]:
            color = hex_to_bgr(kit[f"{kind}_color"])
            team_colors[f"{side}_{kind}"] = color
    if order == "rgb":
        team_colors = {key: value[::-1] for key, value in team_colors.items()}
    return team_colors


def get_group_to_color(match, order="bgr", kind="jersey"):
    """
    Same function as in frontend
    Args:
        * match: dict from the database or saved match.json
    Returns:
        * dict linking group to their color in bgr
    """
    # See readme for integer labels meaning.
    team_colors = get_team_colors(match, order=order)
    home_team_color = team_colors[f"home_{kind}"]
    away_team_color = team_colors[f"away_{kind}"]
    sorted_colors = choose_compatible_colors(
        REFEREE_GOAL_COLORS,
        home_team_color,
        away_team_color,
    )
    group_to_color = {
        "home team": home_team_color,
        "away team": away_team_color,
        "home goalkeeper": sorted_colors[0],
        "away goalkeeper": sorted_colors[1],
        "referee": sorted_colors[2],
        "balls": (255, 255, 255),
    }
    return group_to_color



import json
import os

import click

from get_group_to_color import get_group_to_color
from skcvideo.colors import BLACK, GREEN, WHITE
from skcvideo.minimap import Minimap
from skcvideo.reader import Reader
from skcvideo.timeline import Timeline
from skcvideo.utils import put_text
from skcvideo.video_incrustation import VideoIncrustation


class Visualizator(Reader):
    def __init__(self, video_path, match_data, data, **kwargs):
        self.match_data = match_data
        self.data = data

        self.players = {p["trackable_object"]: p for p in match_data["players"] + match_data["referees"]}
        self.group_to_color = get_group_to_color(self.match_data, order="bgr", kind="jersey")
        self.group_to_text_color = get_group_to_color(self.match_data, order="bgr", kind="number")

        self.video_incrustation = VideoIncrustation(
            box=[0, 575, 720, 575 + 1280],
        )
        self.minimap = Minimap(
            box=[200, 0, 590, 575], pitch_length=match_data["pitch_length"], pitch_width=match_data["pitch_width"]
        )
        self.timeline = Timeline(
            box=[730, 585, 820, 1845],
            parent=self,
            timeline_color=self.timeline_color,
        )

        super().__init__(video_path, **kwargs)

    def info_mapping(self, d):
        trackable_object = d.get("trackable_object", None)
        if trackable_object == 55:
            group = "balls"
        elif "group_name" in d:
            group = d["group_name"]
        else:
            player = self.players[trackable_object]
            if "number" in player:
                if player["team_id"] == self.match_data["home_team"]["id"]:
                    side = "home"
                else:
                    side = "away"

                if player["player_role"]["name"] == "Goalkeeper":
                    kind = "goalkeeper"
                else:
                    kind = "team"

                group = f"{side} {kind}"

            else:
                group = "referee"

        info = {
            "x": d["x"],
            "y": d["y"],
            "radius": 3 if group == "balls" else 7,
            "color": self.group_to_color.get(group, BLACK),
            "second_color": WHITE,
            "text": None,
            "text_color": self.group_to_text_color.get(group, WHITE),
        }
        if trackable_object is not None and trackable_object != 55:
            info["text"] = str(self.players[trackable_object].get("number", "R"))
        return info

    def timeline_color(self, frame):
        if len(self.data[frame]["data"]) > 0:
            return GREEN
        else:
            return BLACK

    def refresh(self):
        put_text(
            img=self.big_image,
            text=f"Frame {self.frame}",
            org=(20, 20),
            align_x="left",
            align_y="top",
        )
        self.timeline.refresh(self.big_image, self.frame)
        self.video_incrustation.refresh(self.big_image, self.image.copy())
        self.minimap.refresh(self.big_image, self.data[self.frame]["data"], self.info_mapping)


@click.command()
@click.option("--data-dir", required=True, help="Path to the data directory")
@click.option("--video-path", default=None, help="Path to the video")
def main(data_dir, video_path):
    match_data_path = os.path.join(data_dir, "match_data.json")
    with open(match_data_path, "rb") as f:
        match_data = json.load(f)
    print("match_id:", match_data["id"])

    data_path = os.path.join(data_dir, "structured_data.json")
    with open(data_path) as f:
        data = json.load(f)

    reader = Visualizator(
        video_path,
        match_data,
        data,
        min_frame=0,
        max_frame=9000,
    )
    reader.start()


if __name__ == "__main__":
    main()



import cv2

from skcvideo.colors import WHITE
from skcvideo.utils import put_text


class ChatBox:
    def __init__(
        self,
        box=None,
        data=None,
        row_height=20,
    ):
        """
        A widget to display text appearing gradually along with the video.
        box: list of coordinates defining the widget delimitation
        data: list of dict
            {
                "frame": (int) frame from which to display the text
                "text": (str) text to be displayed
                "color": (list) or (tuple) of length 3, containing RGB values
                        defines color of the text
            }
        """
        if box is None:
            box = [0, 0, 1920, 1080]
        if data is None:
            data = []
        self.box = box
        self.data = data
        self.row_height = row_height

        x1, y1, x2, y2 = self.box
        self.number_of_rows = int((y2 - y1) / self.row_height)

    def build(self, image):
        x1, y1, x2, y2 = self.box
        image[y1:y2, x1:x2] = 0

        # Draw box
        cv2.rectangle(
            image,
            (x1, y1),
            (x2, y2),
            color=WHITE,
            thickness=2,
        )

    def refresh(self, image, frame):
        x1, y1, x2, y2 = self.box

        # Prepare texts to be displayed
        lines = []
        for d in self.data:
            if d["frame"] <= frame:
                lines.append(d)

        lines = lines[-self.number_of_rows :]
        offset = self.number_of_rows - len(lines)

        # Draw texts
        for i, d in enumerate(lines):
            y = int(round(y1 + (offset + i + 0.5) * self.row_height))
            put_text(
                image,
                d["text"],
                (x1 + 5, y),
                fontScale=0.5,
                align_x="left",
                align_y="center",
                color=d["color"],
            )



import cv2
import numpy as np

from skcvideo.colors import BLACK, RED, WHITE
from skcvideo.core import Button
from skcvideo.utils import put_text


class Timeline:
    """
    A widget representing a timeline which can be used to display some information in color.
    Arguments:
        box (list): coordinates of the delimitation of the widget
        timeline_width (int): width of the timeline in pixels
        margin (int): gap between two lines in pixels
        minute_per_line (int): time represented on one line of the timeline
        ticks (int): intervals used to draw white bars on the timeline
        fps (int): framerate of the information to be displayed
        **kwargs: Additional keyword arguments.
    """

    def __init__(self, box=None, timeline_width=20, margin=5, minute_per_line=3, ticks=10, fps=10, *args, **kwargs):
        if box is None:
            box = [955, 110, 1035, 1910]
        self.name = "Timeline"
        self.box = box
        self.fps = fps
        self.ticks = ticks

        self.timeline_length = self.box[3] - self.box[1]

        self.pixel_per_frame = self.timeline_length / (minute_per_line * 60 * self.fps)

        self.frame_banner_flag = "with_frame_banner" in kwargs
        self.fontsize = kwargs.get("fontsize", 1.5)

        self.timeline_width = timeline_width
        self.margin = margin
        self.gap = self.timeline_width + 2 * self.margin

        self.hitbox = (self.box[1], self.box[0], self.box[3], self.box[2])
        self.buttons = [Button(self.hitbox, self.jump_event)]
        if self.frame_banner_flag:
            self.mouse_position_x = None
            self.mouse_position_y = None
            self.move_mouse_events = [Button(self.hitbox, self.set_mouse_coordinates)]

    @property
    def min_frame(self):
        return getattr(self.parent, "min_frame", 0)

    @property
    def max_frame(self):
        return getattr(self.parent, "max_frame", 9000)

    @property
    def n_timelines(self):
        _, y = self.frame_to_position(self.max_frame)
        return y + 1

    def frames(self):
        """Generator that yields each frame in the timeline."""
        end_x, max_y = self.frame_to_position(self.max_frame)
        end_x = int(np.ceil(end_x))
        for y in range(max_y + 1):
            if y == max_y:
                max_x = end_x
            else:
                max_x = self.timeline_length

            for x in range(max_x + 1):
                frame = self.position_to_frame(x, y)
                if frame >= self.max_frame:
                    return
                yield frame

    def jump_event(self, x, y, *args, **kwargs):
        frame = self.get_frame(x, y)
        self.parent.jump(frame)

    def set_mouse_coordinates(self, x, y, *args, **kwargs):
        self.mouse_position_x, self.mouse_position_y = x, y

    def timeline_color(self, frame):
        """
        Here you define the color of the timeline with repect to the frame.
        """
        return None

    def build(self, image):
        """
        Draws the timeline's background composed of several timeline lines
        with box and graduations.
        """
        # Puts time labels label
        put_text(
            img=image,
            text="min",
            org=(self.box[1] - 90, self.box[0] - self.margin - self.timeline_width // 2),
            fontScale=0.6,
            align_x="left",
        )

        # Draws graduations labels
        for x in range(0, self.timeline_length):
            frame = self.position_to_frame(x, 0)
            second = frame / self.fps

            if second % 60 == 0:
                second = int(second)
                put_text(
                    img=image,
                    text=f"{second // 60}min",
                    org=(self.box[1] + x, self.box[0] - self.margin - self.timeline_width // 2 - 2),
                    fontScale=0.6,
                    align_x="center",
                )
            elif second % self.ticks == 0:
                second = int(second)
                put_text(
                    img=image,
                    text=f"{second % 60}s",
                    org=(self.box[1] + x, self.box[0] - self.margin - self.timeline_width // 2 + 4),
                    fontScale=0.4,
                    align_x="center",
                )

        # Draws each timeline's line
        for i in range(self.n_timelines):
            self.draw_timeline_box(image, i)

        # Draws graduations
        for frame in range(self.min_frame, self.max_frame):
            x, y = self.frame_to_position(frame)
            x = int(round(x))
            second = frame / self.fps

            # A small mark every 5 seconds
            if (second % 5) == 0:
                cv2.line(
                    image,
                    (self.box[1] + x, self.box[0] + y * self.gap + self.margin - 1),
                    (self.box[1] + x, self.box[0] + (y + 1) * self.gap - self.margin + 1),
                    color=WHITE,
                    thickness=1,
                )

            # A big mark every minute
            if (second % 60) == 0:
                cv2.line(
                    image,
                    (self.box[1] + x, self.box[0] + y * self.gap + self.margin - 3),
                    (self.box[1] + x, self.box[0] + (y + 1) * self.gap - self.margin + 3),
                    color=WHITE,
                    thickness=2,
                )

        self.draw_timeline_data(image)

    def refresh(self, image, frame):
        self.draw_timer(image, frame)

    def draw_timeline_box(self, image, y):
        """
        Draws one line of the timeline's background, which consists in a
        simple white box.
        """
        # Manage the offset
        y_min = self.box[0] + self.margin + y * self.gap
        y_max = y_min + self.timeline_width

        max_x, max_y = self.frame_to_position(self.max_frame)
        max_x = int(np.ceil(max_x))

        # The last box may not go up to the end.
        if y == max_y:
            timeline_max = self.box[1] + max_x
        else:
            timeline_max = self.box[3]

        # Draws the box
        cv2.rectangle(
            image,
            (self.box[1], y_min),
            (timeline_max, y_max),
            color=WHITE,
            thickness=1,
        )

        # Adds a time label before the line
        min_frame = self.position_to_frame(0, y)
        min_second = min_frame / self.fps
        min_minute = int(np.floor(min_second / 60))
        max_frame = self.position_to_frame(self.timeline_length, y)
        max_second = max_frame / self.fps
        max_minute = int(np.ceil(max_second / 60))
        put_text(
            img=image,
            text=f"{min_minute}-{max_minute}",
            org=(self.box[1] - 90, (y_min + y_max) // 2),
            fontScale=0.6,
            align_x="left",
        )

    def draw_timeline_data(self, im):
        """
        Draws information on the timeline. Useful to have a global view of
        your data or to have a reference for jumping in the video.
        """
        for frame in self.frames():
            color = self.timeline_color(frame)
            self.draw_one_timeline_data(im, frame, color)

    def draw_one_timeline_data(self, im, frame, color):
        """
        Colors the given frame on the timeline according to the timeline_color
        function
        """
        x, y = self.frame_to_position(frame)
        x = int(round(x))
        if color is not None:
            x1 = self.box[1] + x
            x2 = x1 + int(np.ceil(self.pixel_per_frame))
            y1 = y * self.gap + self.box[0] + self.margin + 1
            y2 = y1 + self.timeline_width - 2
            im[y1:y2, x1:x2] = color

    def draw_timer(self, image, frame):
        """
        Draws a timer on the timeline on the given frame. To be used in the
        refresh method of the parent Reader class.
        """
        timer_x, timer_y = self.frame_to_position(frame)
        timer_x = int(round(timer_x))
        cv2.line(
            image,
            (self.box[1] + timer_x, timer_y * self.gap + self.box[0]),
            (self.box[1] + timer_x, (timer_y + 1) * self.gap + self.box[0]),
            color=WHITE,
            thickness=3,
        )
        cv2.line(
            image,
            (self.box[1] + timer_x, timer_y * self.gap + self.box[0]),
            (self.box[1] + timer_x, (timer_y + 1) * self.gap + self.box[0]),
            color=RED,
            thickness=2,
        )
        if self.frame_banner_flag and self.mouse_position_x is not None:
            frame_on_mouse = self.get_frame(self.mouse_position_x, self.mouse_position_y)
            if frame_on_mouse > 0:
                self.draw_frame_banner_on_image(image, frame_on_mouse, self.mouse_position_x, self.mouse_position_y)

    def draw_frame_banner_on_image(self, image, number, x, y):
        """
        Draws a banner with the given frame on the timeline.
        """
        number_str = str(number)

        alpha = 0.7
        overlay = image.copy()

        (text_height, text_width), _ = cv2.getTextSize(
            text=number_str, fontFace=cv2.FONT_HERSHEY_SIMPLEX, fontScale=self.fontsize, thickness=2
        )

        banner_width = text_width + 10
        banner_height = text_height + 5

        if x + banner_height > self.box[3]:
            rect_start, rect_end = (x - banner_height, y), (x, y - banner_width)
            text_x = x - banner_height // 2
        else:
            rect_start, rect_end = (x, y), (x + banner_height, y - banner_width)
            text_x = x + banner_height // 2

        text_y = y - banner_width // 2

        cv2.rectangle(overlay, rect_start, rect_end, WHITE, -1)
        cv2.addWeighted(overlay, alpha, image, 1 - alpha, 0, image)

        put_text(
            image,
            number_str,
            (text_x, text_y),
            color=BLACK,
            thickness=2,
            fontScale=self.fontsize,
            align_x="center",
            align_y="center",
        )

    def get_frame(self, x, y):
        """
        Returns the frame corresponding to a given pixel of the timeline.
        It is used to be able to click one the timeline to jump to a
        particular frame.
        """
        x = x - self.box[1]
        y = (y - self.box[0]) // self.gap
        frame = self.position_to_frame(x, y)
        return frame

    def position_to_frame(self, x, y):
        """
        Returns the frame corresponding to a given pixel of the timeline.
        x: position on one line of the timeline
        y: line number
        """
        frame = (x + y * self.timeline_length) / self.pixel_per_frame + self.min_frame
        frame = int(round(frame))
        return frame

    def frame_to_position(self, frame):
        """
        Returns the position of a given frame on the timeline.
        x: position on one line of the timeline
        y: line number
        """
        timer = (frame - self.min_frame) * self.pixel_per_frame
        x = timer % self.timeline_length
        y = timer // self.timeline_length
        y = int(y)
        return x, y


GRAY_BAR = np.array(
    [
        [189, 189, 190],
        [193, 193, 194],
        [196, 197, 198],
        [200, 201, 202],
        [204, 205, 206],
        [208, 208, 209],
        [212, 212, 213],
        [215, 216, 217],
        [219, 220, 221],
        [235, 235, 236],
    ],
)

BLUE_BAR = np.array(
    [
        [224, 137, 44],
        [218, 134, 43],
        [213, 130, 42],
        [207, 127, 41],
        [204, 125, 40],
        [204, 125, 40],
        [204, 125, 40],
        [204, 125, 40],
    ],
)


class VlcTimeline:
    def __init__(self, box=None):
        """
        Args:
            box: list [x1, y1, x2, y2]
        """
        if box is None:
            box = [79, 966, 1771, 976]
        self.box = box
        self.buttons = [Button(self.box, self.jump_event)]
        self.timeline_length = float(self.box[2] - self.box[0])

    @property
    def min_frame(self):
        return getattr(self.parent, "min_frame", 0)

    @property
    def max_frame(self):
        return getattr(self.parent, "max_frame", 9000)

    @property
    def frames_length(self):
        return float(self.max_frame - self.min_frame)

    def jump_event(self, x, y, *kwargs):
        frame = self.get_frame(x, y)
        self.parent.jump(frame)

    def build(self, image):
        image[self.box[1] : self.box[1] + 18] = np.array([240, 241, 242])[np.newaxis, np.newaxis, :]
        image[self.box[1] + 4 : self.box[1] + 14, self.box[0] : self.box[2]] = GRAY_BAR[:, np.newaxis, :]

    def refresh(self, image, frame):
        self.draw_timer(image, frame)

    def draw_timer(self, image, frame):
        frame = float(frame - self.min_frame)
        i = int(np.round(frame / self.frames_length * self.timeline_length))
        image[self.box[1] + 5 : self.box[1] + 13, self.box[0] : self.box[0] + i] = BLUE_BAR[:, np.newaxis, :]

    def get_frame(self, x, y):
        x = float(x - self.box[0])
        frame = int(np.round(x / self.timeline_length * self.frames_length)) + self.min_frame
        return frame



import matplotlib.cm as cm
import numpy as np


class Colormap:
    def __init__(self, value_green=0.0, value_red=1.0):
        """
        A colormap utility that maps scalar values to RGB colors using a
        modified 'jet' colormap.
        Methods:
            get(value): Returns the RGB color corresponding to the input value,
                        scaled between value_green and value_red.
        """

        self.value_green = value_green
        self.value_red = value_red

        self.colormap = cm.jet(np.linspace(0, 1, 200))
        self.colormap = self.colormap[100:][:, [2, 1, 0]]
        self.colormap = np.round(self.colormap * 255).tolist()

    def get(self, value):
        color_i = int((value - self.value_green) / (self.value_red - self.value_green) * 100.0)
        if color_i < 0:
            color_i = 0
        if color_i >= 100:
            color_i = 99
        color = self.colormap[color_i]
        return color



import numpy as np

CIRCLE_RADIUS = 9.15
GOAL_SIZE = 7.32
GOAL_AREA_SIZE = 5.5
PENALTY_AREA_SIZE = 16.5
PENALTY_MARK_DISTANCE = 11.0


# This file contains all the information concerning the soccer field model.


def create_field_objects(pitch_length=105.0, pitch_width=68.0):
    arc_angle = np.arccos((PENALTY_AREA_SIZE - PENALTY_MARK_DISTANCE) / CIRCLE_RADIUS)
    field_objects = {
        "Center Circle": {
            "type": "circle",
            "x": 0.0,
            "y": 0.0,
            "radius": CIRCLE_RADIUS,
            "startAngle": 0.0,
            "endAngle": 2.0 * np.pi,
        },
        "Halfway line": {
            "type": "line",
            "start_point": (0.0, -pitch_width / 2.0),
            "end_point": (0.0, pitch_width / 2.0),
        },
    }

    for level, level_sign in zip(["Lower", "Upper"], [-1.0, 1.0]):
        x = pitch_length / 2.0
        y = pitch_width / 2.0
        field_objects[f"{level} touch line"] = {
            "type": "line",
            "start_point": (
                -x,
                level_sign * y,
            ),
            "end_point": (
                x,
                level_sign * y,
            ),
        }

    for side, sign in zip(["Right", "Left"], [1.0, -1.0]):
        x = pitch_length / 2.0
        y = pitch_width / 2.0
        field_objects[f"{side} goal line"] = {
            "type": "line",
            "start_point": (
                sign * x,
                -y,
            ),
            "end_point": (
                sign * x,
                y,
            ),
        }

        x = pitch_length / 2.0 - PENALTY_AREA_SIZE
        y = GOAL_SIZE / 2.0 + PENALTY_AREA_SIZE
        field_objects[f"{side} penalty line"] = {
            "type": "line",
            "start_point": (
                sign * x,
                -y,
            ),
            "end_point": (
                sign * x,
                y,
            ),
        }

        x = pitch_length / 2.0 - GOAL_AREA_SIZE
        y = GOAL_SIZE / 2.0 + GOAL_AREA_SIZE
        field_objects[f"{side} goal area line"] = {
            "type": "line",
            "start_point": (
                sign * x,
                -y,
            ),
            "end_point": (
                sign * x,
                y,
            ),
        }

        field_objects[f"{side} penalty arc"] = {
            "type": "circle",
            "x": sign * (pitch_length / 2.0 - PENALTY_MARK_DISTANCE),
            "y": 0.0,
            "radius": CIRCLE_RADIUS,
            "startAngle": -arc_angle - (sign + 1.0) * np.pi / 2.0,
            "endAngle": arc_angle - (sign + 1.0) * np.pi / 2.0,
        }

        for level, level_sign in zip(["lower", "upper"], [-1.0, 1.0]):
            y = GOAL_SIZE / 2.0 + PENALTY_AREA_SIZE
            field_objects[f"{side} {level} penalty line"] = {
                "type": "line",
                "start_point": (
                    sign * pitch_length / 2.0,
                    level_sign * y,
                ),
                "end_point": (
                    sign * (pitch_length / 2.0 - PENALTY_AREA_SIZE),
                    level_sign * y,
                ),
            }

            y = GOAL_SIZE / 2.0 + GOAL_AREA_SIZE
            field_objects[f"{side} {level} goal area line"] = {
                "type": "line",
                "start_point": (
                    sign * pitch_length / 2.0,
                    level_sign * y,
                ),
                "end_point": (
                    sign * (pitch_length / 2.0 - GOAL_AREA_SIZE),
                    level_sign * y,
                ),
            }
    return field_objects



import os
import unittest
from unittest.mock import patch

import cv2
from ffmphisdp.utils import create_video, decode_all_frames, expected_frame_color, is_almost_same_color

from skcvideo.controlled_fps_video_capture import ControlledFPSVideoCapture


class FakeCapture:
    def __init__(self, *args, **kwargs):
        self.input_fps = kwargs.get("input_fps")

    def get(self, prop):
        return self.input_fps

    def read(self):
        return True, None


class TestControlledFPSVideoCapture(unittest.TestCase):
    def test_reader_selection(self):
        """This test validate that when setting the frame_selection_version we get the correct reader"""
        from ffmphisdp.pyav_reader import ControlledVideoReader as ControlledFPSVideoCapture_ffmphisdp

        from skcvideo.controlled_fps_video_capture import ControlledFPSVideoCapture_skcvideo

        reader = ControlledFPSVideoCapture()
        self.assertIsInstance(reader, ControlledFPSVideoCapture_skcvideo)
        self.assertEqual(reader.frame_selection_version, ControlledFPSVideoCapture.FRAME_SELECTION_LEGACY)

        reader = ControlledFPSVideoCapture(frame_selection_version=ControlledFPSVideoCapture.FRAME_SELECTION_LEGACY)
        self.assertIsInstance(reader, ControlledFPSVideoCapture_skcvideo)
        self.assertEqual(reader.frame_selection_version, ControlledFPSVideoCapture.FRAME_SELECTION_LEGACY)

        filename = "test.mp4"
        create_video([(filename, 25, 50)], filename)
        all_frames = decode_all_frames(filename, framerate=10, use_gpu=False)
        frame_infos = list(all_frames.keys())
        frame_selection = list(all_frames.values())

        reader = ControlledFPSVideoCapture(
            filename,
            frame_infos=frame_infos,
            frame_selection=frame_selection,
            frame_selection_version=ControlledFPSVideoCapture.FRAME_SELECTION_FFMPHISDP,
        )
        self.assertIsInstance(reader, ControlledFPSVideoCapture_ffmphisdp)
        self.assertEqual(reader.frame_selection_version, ControlledFPSVideoCapture.FRAME_SELECTION_FFMPHISDP)
        os.remove(filename)

    @patch("cv2.VideoCapture", side_effect=FakeCapture)
    def test_controlled_fps_video_capture(self, *args):
        expected_results = {
            10: [0, 1, 2, 3, 4],
            25: [0, 1, 2, 4, 7],
            30: [0, 1, 2, 5, 8],
            50: [0, 1, 4, 9, 14],
            60: [0, 1, 5, 11, 17],
        }
        for input_fps in [10, 25, 30, 50, 60]:
            cap = ControlledFPSVideoCapture(input_fps=input_fps)
            for i in range(5):
                cap.read()
                self.assertEqual(cap.input_frame, expected_results[input_fps][i])

    def test_video_reader_start_frame(self):
        """Test the video reader return the correct frames when starting at a specific frame"""
        # Create the video file
        red_shift = 11
        green_shift = 17
        filename = "test_set_frame_idx.mp4"
        create_video([(filename, 25, 50)], filename, red_shift=red_shift, green_shift=green_shift)

        # Test ControlledFPSVideoCapture at various fps
        fps_test_cases = {
            10: [
                (0, [0, 1, 2, 4, 7, 9, 12, 14, 17, 19, 22, 24, 27, 29, 32]),
                (1, [1, 2, 4, 7, 9, 12, 14, 17, 19, 22, 24, 27, 29, 32]),
                (2, [2, 4, 7, 9, 12, 14, 17, 19, 22, 24, 27, 29, 32]),
                (3, [4, 7, 9, 12, 14, 17, 19, 22, 24, 27, 29, 32]),
                (4, [7, 9, 12, 14, 17, 19, 22, 24, 27, 29, 32]),
                (5, [9, 12, 14, 17, 19, 22, 24, 27, 29, 32]),
            ],
            25: [
                (0, [0, 1, 2, 3, 4, 5]),
                (1, [1, 2, 3, 4, 5, 6]),
                (2, [2, 3, 4, 5, 6, 7]),
                (3, [3, 4, 5, 6, 7, 8]),
            ],
            5: [
                (0, [0, 1, 4, 9, 14, 19, 24]),
                (1, [1, 4, 9, 14, 19, 24]),
                (2, [4, 9, 14, 19, 24]),
                (3, [9, 14, 19, 24]),
                (4, [14, 19, 24]),
                (5, [19, 24]),
            ],
        }
        for fps, test_cases in fps_test_cases.items():
            video_reader = ControlledFPSVideoCapture(filename, fps=fps)
            for frame_idx, expected_frames in test_cases:
                with self.subTest((frame_idx, expected_frames)):
                    video_reader.set(cv2.CAP_PROP_POS_FRAMES, frame_idx)
                    # video_reader.set_frame_idx(frame_idx)
                    for expected_frame in expected_frames:
                        ret, frame = video_reader.read()
                        self.assertTrue(ret, msg="reading error")
                        frame_color = frame[0, 0]
                        expected_red, expected_green, _ = expected_frame_color(expected_frame, red_shift, green_shift)
                        self.assertTrue(
                            is_almost_same_color(frame_color[2], expected_red),
                            msg=f"{frame_color[2]} != {expected_red} (Frame_idx {frame_idx}, \
Expected_frame: {expected_frame}, Frame {frame_color}, color: red)",
                        )
                        self.assertTrue(
                            is_almost_same_color(frame_color[1], expected_green),
                            msg=f"{frame_color[1]} != {expected_green} (Frame_idx {frame_idx}, \
Expected_frame: {expected_frame}, Frame {frame_color}, color: green)",
                        )

        # Cleanup
        os.remove(filename)

    def test_set(self):
        # Create the video file
        red_shift = 11
        green_shift = 17
        filename = "test_set_frame_idx.mp4"
        create_video([(filename, 25, 50)], filename, red_shift=red_shift, green_shift=green_shift)
        framerate = 10
        seek_local_frame_idx = 9

        # Test Legacy ControlledFPSVideoCapture at 10 fps
        with self.subTest("Legacy ControlledFPSVideoCapture at 10 fps"):
            video_reader = ControlledFPSVideoCapture(
                filename, fps=framerate, frame_selection_version=ControlledFPSVideoCapture.FRAME_SELECTION_LEGACY
            )
            video_reader.set(cv2.CAP_PROP_POS_FRAMES, seek_local_frame_idx)
            ret, frame = video_reader.read()
            self.assertTrue(ret, msg="reading error")
            frame_color = frame[0, 0]
            expected_global_frame_idx = 19
            expected_red, expected_green, _ = expected_frame_color(expected_global_frame_idx, red_shift, green_shift)
            self.assertTrue(
                is_almost_same_color(frame_color[2], expected_red),
                msg=f"{frame_color[2]} != {expected_red} (Frame_idx {seek_local_frame_idx}, \
Expected_frame: {expected_global_frame_idx}, Frame {frame_color}, color: red)",
            )
            self.assertTrue(
                is_almost_same_color(frame_color[1], expected_green),
                msg=f"{frame_color[1]} != {expected_green} (Frame_idx {seek_local_frame_idx}, \
Expected_frame: {expected_global_frame_idx}, Frame {frame_color}, color: green)",
            )

        # Test ffmphisdp ControlledFPSVideoCapture at 10 fps
        with self.subTest("ffmphisdp ControlledFPSVideoCapture at 10 fps"):
            all_frames = decode_all_frames(filename, framerate=10, use_gpu=False)
            frame_infos = list(all_frames.keys())
            frame_selection = list(all_frames.values())
            print(frame_infos)
            print(frame_selection)
            video_reader = ControlledFPSVideoCapture(
                filename,
                frame_infos=frame_infos,
                frame_selection=frame_selection,
                frame_selection_version=ControlledFPSVideoCapture.FRAME_SELECTION_FFMPHISDP,
            )
            video_reader.set(cv2.CAP_PROP_POS_FRAMES, seek_local_frame_idx)
            ret, frame = video_reader.read()
            self.assertTrue(ret, msg="reading error")
            frame_color = frame[0, 0]
            expected_global_frame_idx = 23
            expected_red, expected_green, _ = expected_frame_color(expected_global_frame_idx, red_shift, green_shift)
            self.assertTrue(
                is_almost_same_color(frame_color[2], expected_red),
                msg=f"{frame_color[2]} != {expected_red} (Frame_idx {seek_local_frame_idx}, \
Expected_frame: {expected_global_frame_idx}, Frame {frame_color}, color: red)",
            )
            self.assertTrue(
                is_almost_same_color(frame_color[1], expected_green),
                msg=f"{frame_color[1]} != {expected_green} (Frame_idx {seek_local_frame_idx}, \
Expected_frame: {expected_global_frame_idx}, Frame {frame_color}, color: green)",
            )

    def test_get_ffmphisdp(self):
        """Test the get method of ControlledFPSVideoCapture_ffmphisdp"""
        # Create the video file
        red_shift = 11
        green_shift = 17
        filename = "test_set_frame_idx.mp4"
        create_video([(filename, 25, 50)], filename, red_shift=red_shift, green_shift=green_shift)
        all_frames = decode_all_frames(filename, framerate=10, use_gpu=False)
        frame_infos = list(all_frames.keys())
        frame_selection = list(all_frames.values())
        print(frame_infos)
        print(frame_selection)
        video_reader = ControlledFPSVideoCapture(
            filename,
            frame_infos=frame_infos,
            frame_selection=frame_selection,
            frame_selection_version=ControlledFPSVideoCapture.FRAME_SELECTION_FFMPHISDP,
        )
        seek_local_frame_idx = 9  # global_idx = 23
        video_reader.set(cv2.CAP_PROP_POS_FRAMES, seek_local_frame_idx)

        # Read a couple frames
        video_reader.read()  # after read: global_idx = 23 , local = 9
        video_reader.read()  # after read: global_idx = 26 , local = 10
        video_reader.read()  # after read: global_idx = 28 , local = 11

        # Get current_frame
        expected_next_local_frame_idx = 11
        expected_global_index = 28
        self.assertEqual(expected_next_local_frame_idx, video_reader.get(cv2.CAP_PROP_POS_FRAMES))
        self.assertEqual(expected_next_local_frame_idx, video_reader.output_frame)
        self.assertEqual(expected_global_index, video_reader.input_frame)

        # Get frame count
        expected_local_frame_count = 20
        self.assertEqual(expected_local_frame_count, video_reader.get(cv2.CAP_PROP_FRAME_COUNT))



import os
import unittest

from ffmphisdp.utils import create_video, decode_all_frames

from skcvideo.controlled_fps_video_capture import ControlledFPSVideoCapture
from skcvideo.video_capture import TransformedImageVideoCapture
from skcvideo.video_incrustation import VideoIncrustation


class TestTransformedImageVideoCapture(unittest.TestCase):
    def test_reader_selection(self):
        """This test validate that when setting the frame_selection_version we get the correct reader"""
        from ffmphisdp.pyav_reader import ControlledVideoReader as ControlledFPSVideoCapture_ffmphisdp

        from skcvideo.controlled_fps_video_capture import ControlledFPSVideoCapture_skcvideo

        filename = "test.mp4"
        create_video([(filename, 25, 50)], filename)
        all_frames = decode_all_frames(filename, framerate=10, use_gpu=False)
        frame_infos = list(all_frames.keys())
        frame_selection = list(all_frames.values())

        reader = TransformedImageVideoCapture(filename)
        self.assertIsInstance(reader.frame_reader, ControlledFPSVideoCapture_skcvideo)
        self.assertEqual(reader.frame_reader.frame_selection_version, ControlledFPSVideoCapture.FRAME_SELECTION_LEGACY)

        reader = TransformedImageVideoCapture(
            filename, frame_selection_version=ControlledFPSVideoCapture.FRAME_SELECTION_LEGACY
        )
        self.assertIsInstance(reader.frame_reader, ControlledFPSVideoCapture_skcvideo)
        self.assertEqual(reader.frame_reader.frame_selection_version, ControlledFPSVideoCapture.FRAME_SELECTION_LEGACY)

        reader = TransformedImageVideoCapture(
            filename,
            frame_infos=frame_infos,
            frame_selection=frame_selection,
            frame_selection_version=ControlledFPSVideoCapture.FRAME_SELECTION_FFMPHISDP,
        )
        self.assertIsInstance(reader.frame_reader, ControlledFPSVideoCapture_ffmphisdp)
        self.assertEqual(
            reader.frame_reader.frame_selection_version, ControlledFPSVideoCapture.FRAME_SELECTION_FFMPHISDP
        )

        VideoIncrustation(filename, box=[0, 575, 720, 575 + 1280])

        os.remove(filename)



import cv2
import numpy as np

from skcvideo.colors import RED, WHITE
from skcvideo.utils import put_text


class Plot:
    def __init__(
        self,
        box=None,
        margin=100,
        ticks=None,
        unit="m/s",
        values=None,
        min_value=0.0,
        max_value=10.0,
    ):
        """
        Widget for displaying a curve.
        Arguments:
            box: (list) [x1, y1, x2, y2] coordinates of the delimitation of
                 the widget
            margin: (int) size in pixels of the space left before the curve to
                    write the ticks
            ticks: (list) list of values to be used as a scale on the y axis
            unit: (str) unit used for the values e.g. "m/s"
            values: (list) list of float of shape [n_frames] containing values
                    to be plotted
            min_value: (float) value mapped at the bottom of the plot
            max_value: (float) value mapped at the top of the plot
        """
        if box is None:
            box = [960, 930, 1920, 1065]
        self.box = box
        self.m = margin
        self.ticks = ticks
        self.unit = unit
        self.values = values
        self.min_value = min_value
        self.max_value = max_value

    @property
    def x0(self):
        x1, y1, x2, y2 = self.box
        return x1 + self.m

    def get_y(self, value):
        x1, y1, x2, y2 = self.box
        y = (value - self.min_value) / (self.max_value - self.min_value) * (y2 - y1)
        y = int(np.round(y))
        return y2 - y

    def build(self, image):
        x1, y1, x2, y2 = self.box

        # Draw box
        cv2.rectangle(
            image,
            (self.x0, y1),
            (x2, y2),
            color=WHITE,
            thickness=1,
        )

        # Draw ticks
        if self.ticks is not None:
            for tick in self.ticks:
                y = self.get_y(tick)
                cv2.line(
                    image,
                    (self.x0 - 3, y),
                    (self.x0 + 3, y),
                    color=WHITE,
                    thickness=1,
                )
                put_text(
                    img=image,
                    text=f"{tick:.1f} {self.unit}",
                    org=(self.x0 - 10, y),
                    fontScale=0.5,
                    align_x="right",
                    align_y="center",
                )

        # Draw curve
        for frame in range(x2 - self.x0):
            if frame < len(self.values):
                value = self.values[frame]
                if value is not None:
                    y = self.get_y(value)

                    if not (y1 <= y <= y2):
                        continue

                    image[y, self.x0 + frame] = WHITE

    def refresh(self, image, frame):
        x1, y1, x2, y2 = self.box

        # Draw timer
        cv2.rectangle(
            image,
            (self.x0 + frame, y1),
            (self.x0 + frame, y2),
            color=RED,
            thickness=1,
        )

        # Draw current value
        if frame < len(self.values):
            value = self.values[frame]
            if value is not None:
                y = self.get_y(value)
                put_text(
                    img=image,
                    text=f"{value:.2f} {self.unit}",
                    org=(self.x0 + frame + 5, y - 5),
                    fontScale=0.4,
                    align_x="left",
                    align_y="bottom",
                )
